#ifndef TRIANGLE_MESH_HPP
#define TRIANGLE_MESH_HPP

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------		
#include <Triangle.hpp>
#include <string>

namespace Render
{
	//-------------------------------------------------------------------------
	// Class Triangle
	//-------------------------------------------------------------------------
	class TriangleMesh : public Primitive
	{
		public:
			//-----------------------------------------------------------------
			// C/D-Tor
			//-----------------------------------------------------------------
			TriangleMesh(const std::string& _filename)
			{
				const int nTriangles = _filename.size();
				InitTriangles(nTriangles);
			}

			//-----------------------------------------------------------------
			virtual ~TriangleMesh()
			{
				DeleteTriangles();
			}

			//-----------------------------------------------------------------
			// Render primitive
			//-----------------------------------------------------------------
			virtual void Render() const
			{
				for(int i=0;i<nTriangles;++i)
				{
					triangles[i]->Render();
				}
			}

		private:
			//-----------------------------------------------------------------
			// Forbidden operations
			//-----------------------------------------------------------------
			TriangleMesh(const Triangle& _copy);
			TriangleMesh& operator=(const Triangle& _copy);

			//-----------------------------------------------------------------
			// Initialize triangle mesh with n triangles
			//-----------------------------------------------------------------
			void InitTriangles(int _nTriangles)
			{
				if(triangles == NULL)
					DeleteTriangles();

				// Load some factices triangles
				nTriangles = _nTriangles;
				triangles = new Triangle*[nTriangles];
				for(int i=0;i<nTriangles;++i)
				{
					triangles[i] = new Triangle();
				}
			}

			//-----------------------------------------------------------------
			// Delete triangles
			//-----------------------------------------------------------------
			void DeleteTriangles()
			{
				delete triangles;
			}

			//-----------------------------------------------------------------
			// Attributes
			//-----------------------------------------------------------------
			Triangle** triangles;
			int nTriangles;
	};
}

#endif // TRIANGLE_MESH_HPP
