#ifndef TOOLS_ALLOCATOR_HPP
#define TOOLS_ALLOCATOR_HPP

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <MemoryTracer.hpp>

namespace Tools
{
	//-------------------------------------------------------------------------
	// Memory manager definition
	//-------------------------------------------------------------------------
	typedef Tools::MemoryManager<Tools::BasicAllocator,Tools::MemoryTracer> TMemoryManager;
}

//-----------------------------------------------------------------------------
// New / Delete operators definition
//-----------------------------------------------------------------------------
inline void* operator new(std::size_t _size, const char* _file, int _line)
{
	return Tools::TMemoryManager::Instance().Allocate(_size, _file, _line, false);
}
//-----------------------------------------------------------------------------
inline void* operator new[](std::size_t _size, const char* _file, int _line)
{
	return Tools::TMemoryManager::Instance().Allocate(_size, _file, _line, true);
}
//-----------------------------------------------------------------------------
inline void operator delete(void* _pointer)
{
	Tools::TMemoryManager::Instance().Deallocate(_pointer, false);
}
//-----------------------------------------------------------------------------
inline void operator delete[](void* _pointer)
{
	Tools::TMemoryManager::Instance().Deallocate(_pointer, true);
}
//-----------------------------------------------------------------------------
inline void operator delete(void* _pointer, const char* _file, int _line)
{
	Tools::TMemoryManager::Instance().Deallocate(_pointer, false);
}
//-----------------------------------------------------------------------------
inline void operator delete[](void* _pointer, const char* _file, int _line)
{
	Tools::TMemoryManager::Instance().Deallocate(_pointer, true);
}

//-----------------------------------------------------------------------------
// Macro definition
//-----------------------------------------------------------------------------
#define new 	new(__FILE__, __LINE__)
#define delete 	delete


//-----------------------------------------------------------------------------
// Dump memory
//-----------------------------------------------------------------------------
inline void Dump()
{
	Tools::TMemoryManager::Instance().Dump();
}

#endif // TOOLS_ALLOCATOR_HPP
