//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <MemoryTracer.hpp>
#include <iostream>
#include <sstream>
#include <iomanip>

namespace Tools
{
	//--------------------------------------------------------------------------
	// MemoryTracer methods
	//--------------------------------------------------------------------------	
	MemoryTracer& MemoryTracer::Instance()
	{
		static MemoryTracer instance;
		return instance;
	}
	//--------------------------------------------------------------------------	
	MemoryTracer::MemoryTracer()
	{

	}
	//--------------------------------------------------------------------------	
	MemoryTracer::~MemoryTracer()
	{
		dump();
	}
	//--------------------------------------------------------------------------
	void MemoryTracer::TraceAllocation(void* _pointer, size_t _size, const char* _file, int _line, bool _array)
	{
		Instance().traceAllocation(_pointer,_size,_file,_line,_array);
	}
	//--------------------------------------------------------------------------
	void MemoryTracer::TraceDeallocation(void* _pointer, bool _array)
	{
		Instance().traceDeallocation(_pointer,_array);
	}
	//--------------------------------------------------------------------------
	void MemoryTracer::Dump()
	{
		Instance().dump();
	}
	//--------------------------------------------------------------------------
	void MemoryTracer::traceAllocation(void* _pointer, size_t _size, const char* _file, int _line, bool _array)
	{
		// Record block
		MemoryBlock block;
		block.Size       = _size;
		block.File       = _file;
		block.Line       = _line;
		block.Array      = _array;
		blocks[_pointer] = block;
	}
	//--------------------------------------------------------------------------	
	void MemoryTracer::traceDeallocation(void* _pointer, bool _array)
	{
		// Look for memory block
		TMemoryBlockMap::iterator it = blocks.find(_pointer);
		if (it == blocks.end())
		{
			return;
		}

		// Check delete type 
		if (it->second.Array != _array)
		{
			assert(false);
		}

		blocks.erase(it);
	}
	//--------------------------------------------------------------------------	
	void MemoryTracer::dump()
	{
		
		std::cout << "------------------------------------------------\n";
		std::cout << "Dump memory\n";
		std::cout << "------------------------------------------------\n";
		if (blocks.empty())
		{
			std::cout << "No leak detected." << std::endl;
		}
		else
		{
			std::cout << "Some leaks have been detected :" << std::endl;
			std::size_t totalSize = 0;
			TMemoryBlockMap::iterator it;
			for(it = blocks.begin(); it != blocks.end(); ++it)
			{
				// Log memory leak
				totalSize += it->second.Size;
				std::cout << "-> 0x" << reinterpret_cast<int>(it->first)
					   	  << " | " << std::setw(7) << std::setfill(' ')
						  << static_cast<int>(it->second.Size) << " bytes"
						  << " | " << it->second.File
						  << ":" << it->second.Line << std::endl;

				// TODO : Desallocate leak memory ?
			}

			// Log total leak memory
			std::cout << std::endl << std::endl << "-- "
					  << static_cast<int>(blocks.size()) << " non-free blocks, "
					  << static_cast<int>(totalSize) << " bytes --"
					  << std::endl;
		}
	}
}

