#ifndef TOOLS_MEMORY_MANAGER_HPP
#define TOOLS_MEMORY_MANAGER_HPP

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <map>
#include <stack>
#include <string>
#include <cassert>

namespace Tools
{	

	//-------------------------------------------------------------------------
	// Policies
	//-------------------------------------------------------------------------
	//-------------------------------------------------------------------------
	class BasicAllocator
	{
		public:
			static inline void* Allocate(size_t _size) { void* m=malloc(_size); assert(m!=NULL); return m; };
			static inline void Deallocate(void* _ptr) { free(_ptr); };
	};
	//-------------------------------------------------------------------------
	class MemoryTracer
	{
		public:
			//-----------------------------------------------------------------
			// Constructor
			//-----------------------------------------------------------------
			static MemoryTracer& Instance();
			~MemoryTracer();

			//-----------------------------------------------------------------
			// Methods
			//-----------------------------------------------------------------
			static void TraceAllocation(void* _pointer, size_t _size, const char* _file, int _line, bool _array);
			static void TraceDeallocation(void* _pointer, bool _array);
			static void Dump();

		private:
			//-----------------------------------------------------------------
			// Forbidden methods
			//-----------------------------------------------------------------
			MemoryTracer();
			MemoryTracer(const MemoryTracer& _copy);
			MemoryTracer& operator=(const MemoryTracer& _copy);

			//-----------------------------------------------------------------
			// Methods
			//-----------------------------------------------------------------
			void traceAllocation(void* _pointer, size_t _size, const char* _file, int _line, bool _array);
			void traceDeallocation(void* _pointer, bool _array);
			void dump();

			//-----------------------------------------------------------------
			// Attributes
			//-----------------------------------------------------------------
			struct MemoryBlock
			{
				std::size_t  						Size;  // Allocate size
				std::string  						File;  // Filename of allocation
				int          						Line;  // Line of allocation
				bool         						Array; // Element/Array allocation
			};
			//-----------------------------------------------------------------
			typedef std::map<void*,MemoryBlock> 	TMemoryBlockMap;
			typedef std::stack<MemoryBlock> 		TMemoryBlockStack;
			//-----------------------------------------------------------------
			TMemoryBlockMap 						blocks;
	};


	//-------------------------------------------------------------------------
	// Class MemoryManager
	//-------------------------------------------------------------------------
	template<class AllocationPolicy, class TracePolicy>
	class MemoryManager
	{
		public:
			//------------------------------------------------------------------
			// Constructor
			//------------------------------------------------------------------
			~MemoryManager();

			//------------------------------------------------------------------
			// Methods
			//------------------------------------------------------------------
			static MemoryManager& Instance();
			void*  Allocate(size_t _size, const char* _file, int _line, bool _array);
			void   Deallocate(void* _pointer, bool _array);

			void   Dump();

		private:
			//------------------------------------------------------------------
			// Forbidden methods
			//------------------------------------------------------------------
			MemoryManager();
			MemoryManager(const MemoryManager& _copy);
			MemoryManager& operator=(const MemoryManager& _copy);
	}; 
}

//-----------------------------------------------------------------------------
// Includes inline definition
//-----------------------------------------------------------------------------
#include <MemoryTracer.inl>

#endif // TOOLS_MEMORY_MANAGER_HPP
