//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------		
#include <Scene.hpp>
#include <Sphere.hpp>
#include <Triangle.hpp>
#include <TriangleMesh.hpp>
#include <Accelerator.hpp>
#include <cstdio>

using namespace Render;

//-----------------------------------------------------------------------------		
// Operators globaux
//-----------------------------------------------------------------------------		
void* operator new(std::size_t _size)
{
	printf("Global::new\n");
	void* pointer = malloc(_size);
	if(pointer == NULL)
		throw std::bad_alloc();
	return pointer;
}
//-----------------------------------------------------------------------------		
void operator delete(void* _pointer)
{
	printf("Global::delete\n");
	free(_pointer);		
}


//-----------------------------------------------------------------------------		
// Expected trace
//
// Triangle::new
// Triangle::delete
//-----------------------------------------------------------------------------		
void test1()
{
	Triangle* triangle = new Triangle();
	delete triangle;
}

//-----------------------------------------------------------------------------		
// Expected trace
//
// Sphere::new[]
// Sphere::delete[]
//-----------------------------------------------------------------------------		
void test2()
{
	Sphere* sphere = new Sphere[10];
	delete[] sphere;
}

//-----------------------------------------------------------------------------		
// Expected trace
//
// Triangle::new
// Sphere::new
// Global::new
// Global::new
// Global::new
// Triangle::Render()
// Sphere::Render()
// Triangle::delete
// Sphere::delete
// Global::delete
// Global::delete
// Global::delete
// 
//-----------------------------------------------------------------------------		
void test3()
{
	Triangle* triangle = new Triangle();
	Sphere* sphere = new Sphere();

	Aggregate* aggregate = new Aggregate();
	aggregate->Add(triangle);
	aggregate->Add(sphere);
	Scene scene(aggregate);
	scene.Render();
}

//-----------------------------------------------------------------------------		
// Expected trace
//
// Triangle::new placement
//-----------------------------------------------------------------------------	
void test4()
{
	// new placement
	unsigned char memory[sizeof(Triangle)];

	Triangle* triangle = new(memory) Triangle();
	triangle->~Triangle();
}

//-----------------------------------------------------------------------------		
// Expected trace
// 
// Triangle::new (line:84, file:main.cpp)
// Triangle::delete
//-----------------------------------------------------------------------------	
void test5()
{
	Triangle* triangle = new(__LINE__,__FILE__) Triangle();
	delete triangle;
}

int main()
{
	printf("Test 1 ----------------------------------------------------\n");
	test1();
	printf("Test 2 ----------------------------------------------------\n");
	test2();
	printf("Test 3 ----------------------------------------------------\n");
	test3();
	printf("Test 4 ----------------------------------------------------\n");
	test4();
	printf("Test 5 ----------------------------------------------------\n");
	test5();
	return 0;
}
