#ifndef MEMORY_POOL_HPP
#define MEMORY_POOL_HPP

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------		
#include <map>

namespace Tools
{
	//-------------------------------------------------------------------------
	// Class MemoryPool
	//-------------------------------------------------------------------------
	template<typename T>
	class MemoryPool
	{
		public:
			//-----------------------------------------------------------------
			// C/D-Tor
			//-----------------------------------------------------------------
			MemoryPool(std::size_t _numBlockPerChunk=128);
			~MemoryPool();

			//-----------------------------------------------------------------
			// Allocates a block of fixed size
			//-----------------------------------------------------------------
			void* Allocate();

			//-----------------------------------------------------------------
			// Deallocates a block previously allocated with Allocate
			// (undefined behavior if called with the wrong pointer)
			//-----------------------------------------------------------------
			void Desallocate(void* _ptr);

		private:
			//-----------------------------------------------------------------
			// Forbidden operations
			//-----------------------------------------------------------------
			MemoryPool(const MemoryPool& _copy);
			MemoryPool& operator=(const MemoryPool& _copy);

			//-----------------------------------------------------------------
			// Class Chunk
			//-----------------------------------------------------------------
			struct Chunk
			{ 
				typedef unsigned char Byte;
				void Initialize(std::size_t _blockSize, Byte _blocks);
				void Release();
				void* Allocate(std::size_t _blockSize); 
				void Deallocate(void* _ptr, std::size_t _blockSize); 
				Byte* Data; 
				Byte NestAvailableBlock, BlocksAvailable;
			}; 

			//-----------------------------------------------------------------
			// Attributes
			//-----------------------------------------------------------------
			typedef unsigned int TKey;
			typedef std::map<TKey, Chunk> TChunkCollection;
			std::size_t blockSize;
			std::size_t fullBlockSize;
			unsigned char numBlocks; 
			TChunkCollection chunks;
			Chunk* currentChunk;
			TKey currentID;
	};
}

//-----------------------------------------------------------------------------
// Includes inline definition
//-----------------------------------------------------------------------------	
#include "FastMemoryPool.inl"

#endif // MEMORY_POOL_HPP
