//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <cassert>

namespace Tools
{
	//-------------------------------------------------------------------------
	// Memory Pool Definition
	//-------------------------------------------------------------------------
	template<typename T>
	MemoryPool<T>::MemoryPool(std::size_t _numBlockPerChunk):
	blockSize(sizeof(T)),
	fullBlockSize(sizeof(TKey) + sizeof(T)),
	numBlocks(_numBlockPerChunk),
	currentChunk(NULL),
	currentID(0)
	{
		chunks[currentID] = Chunk(numBlocks,fullBlockSize);
	}
	//-------------------------------------------------------------------------
	template<typename T>
	MemoryPool<T>::~MemoryPool()
	{
		typename TChunkCollection::iterator it = chunks.begin();
		for (; it != chunks.end(); ++it)
		{
		   assert(it->second.BlocksAvailable == numBlocks); // Non free memory !
		   it->Release();
		}
	}
	//-------------------------------------------------------------------------
	template<typename T>
	void* MemoryPool<T>::Allocate() 
	{ 
		totalMem = sizeof(int) + blockSize;

		if(currentChunk->BlocksAvailable==0)
		{
			TKey id = chunks.size();
			chunks[id] = Chunk(numBlocks,fullBlockSize);
			currentChunk = &chunks[id];
		}

		void* ptr = currentChunk->Allocate(fullBlockSize);
		*ptr = currentID;
		return ptr+sizeof(TKey);
	}
	//-------------------------------------------------------------------------
	template<typename T>
	void MemoryPool<T>::Desallocate(void* _ptr)
	{ 
		TKey id = currentChunk-sizeof(TKey);
		Chunk* chunk = &chunks[id];
		chunk->Deallocate(_ptr,_blockSize)
		if(chunk->BlocksAvailable == numBlocks)
			chunks.remove(id);
	}


	//-------------------------------------------------------------------------
	// Chunk definition
	//-------------------------------------------------------------------------
	template<typename T>
	void MemoryPool<T>::Chunk::Initialize(std::size_t _blockSize, Byte _blocks)
	{
		assert(_blockSize > 0);
		assert(_blocks > 0);

		Data = new Byte[_blockSize * _blocks]; 
		NestAvailableBlock = 0;
		BlocksAvailable = _blocks;
	} 
	//-------------------------------------------------------------------------
	template<typename T>
	void MemoryPool<T>::Chunk::Release()
	{ 
		delete[] Data;
	} 
	//-------------------------------------------------------------------------
	template<typename T>
	void* MemoryPool<T>::Chunk::Allocate(std::size_t _blockSize) 
	{ 
		if (!BlocksAvailable) return 0; 

		Byte* ptr = Data + (NextAvailableBlock++*_blockSize); 
		--BlocksAvailable;
		return ptr; 
	}
	//-------------------------------------------------------------------------
	template<typename T>
	void MemoryPool<T>::Chunk::Deallocate(void* _ptr, std::size_t _blockSize) 
	{ 
		assert(_ptr >= Data);
		++BlocksAvailable; 
	} 
}
