#ifndef MEMORY_POOL_HPP
#define MEMORY_POOL_HPP

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------		
#include <vector>

namespace Tools
{
	//-------------------------------------------------------------------------
	// Class MemoryPool
	//-------------------------------------------------------------------------
	template<typename T>
	class MemoryPool
	{
		public:
			//-----------------------------------------------------------------
			// C/D-Tor
			//-----------------------------------------------------------------
			MemoryPool(std::size_t _numBlockPerChunk=128);
			~MemoryPool();

			//-----------------------------------------------------------------
			// Allocates a block of fixed size
			//-----------------------------------------------------------------
			void* Allocate();

			//-----------------------------------------------------------------
			// Deallocates a block previously allocated with Allocate
			// (undefined behavior if called with the wrong pointer)
			//-----------------------------------------------------------------
			void Desallocate(void* _ptr);

		private:
			//-----------------------------------------------------------------
			// Forbidden operations
			//-----------------------------------------------------------------
			MemoryPool(const MemoryPool& _copy);
			MemoryPool& operator=(const MemoryPool& _copy);

			//-----------------------------------------------------------------
			// Class Chunk
			//-----------------------------------------------------------------
			struct Chunk 
			{ 
				typedef unsigned char Byte;
				void Initialize(std::size_t _blockSize, Byte _blocks);
				void Release();
				void* Allocate(std::size_t _blockSize); 
				void Deallocate(void* _ptr, std::size_t _blockSize); 
				Byte* Data; 
				Byte FirstAvailableBlock, BlocksAvailable; 
			}; 

			//-----------------------------------------------------------------
			// Desallocate pointer 
			//-----------------------------------------------------------------
			void DoDeallocate(void* _ptr);

			//-----------------------------------------------------------------
			// Find chunk where _ptr comes from
			//-----------------------------------------------------------------
			Chunk* Find(void* _ptr);

			//-----------------------------------------------------------------
			// Attributes
			//-----------------------------------------------------------------
			typedef std::vector<Chunk> TChunkCollection; 
			std::size_t blockSize;
			unsigned char numBlocks; 
			TChunkCollection chunks; 
			Chunk* allocationChunk; 
			Chunk* deallocationChunk; 
	};
}

//-----------------------------------------------------------------------------
// Includes inline definition
//-----------------------------------------------------------------------------	
#include "MemoryPool.inl"

#endif // MEMORY_POOL_HPP
