//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "MemoryPool.hpp"

//-----------------------------------------------------------------------------		
// Constants
//-----------------------------------------------------------------------------
const int NUM_ALLOCATIONS = 10;

//-----------------------------------------------------------------------------		
// Example class
//-----------------------------------------------------------------------------	
struct Example
{

	//-------------------------------------------------------------------------
	// C-Dtor
	//-------------------------------------------------------------------------
	Example()
	{
		printf("Example::Example()\n");
	}
	//-------------------------------------------------------------------------
	~Example()
	{
		printf("Example::~Example()\n");
	}


	//-------------------------------------------------------------------------
	// Operator
	//-------------------------------------------------------------------------
	void* operator new(std::size_t _size)
	{
		return Example::Pool.Allocate();
	}
	//-------------------------------------------------------------------------
	void operator delete(void* _pointer)
	{
		Example::Pool.Desallocate(_pointer);
	}

	//-------------------------------------------------------------------------
	// Attributes
	//-------------------------------------------------------------------------
	int value;
	char stuff0[4];
	float stuff1;

	static Tools::MemoryPool<Example> Pool;
};

Tools::MemoryPool<Example> Example::Pool;


//-----------------------------------------------------------------------------		
// Expected trace
// 
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// MemoryPool::Allocate
// value : 0
// value : 1
// value : 2
// value : 3
// value : 4
// value : 5
// value : 6
// value : 7
// value : 8
// value : 9
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate
// MemoryPool::Deallocate

//-----------------------------------------------------------------------------	
void test1()
{
	Tools::MemoryPool<Example> pool;		
	Example* a[NUM_ALLOCATIONS];

	for(int i=0; i<NUM_ALLOCATIONS; ++i)
	{
		a[i] = static_cast<Example*>(pool.Allocate());
		a[i]->value = i;
	}

	for(int i=0; i<NUM_ALLOCATIONS; ++i)
	{
		printf("value : %d\n", a[i]->value);
	}

	for(int i=0; i<NUM_ALLOCATIONS; ++i)
	{
		pool.Desallocate(static_cast<void*>(a[i]));
	}
}


//-----------------------------------------------------------------------------
// Expected trace
//
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// MemoryPool::Allocate
// Example::Example()
// value : 0
// value : 1
// value : 2
// value : 3
// value : 4
// value : 5
// value : 6
// value : 7
// value : 8
// value : 9
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
// Example::~Example()
// MemoryPool::Deallocate
//-----------------------------------------------------------------------------		
void test2()
{	
	Example* a[NUM_ALLOCATIONS];

	for(int i=0; i<NUM_ALLOCATIONS; ++i)
	{
		a[i] = new Example();
		a[i]->value = i;
	}

	for(int i=0; i<NUM_ALLOCATIONS; ++i)
	{
		printf("value : %d\n", a[i]->value);
	}

	for(int i=0; i<NUM_ALLOCATIONS; ++i)
	{
		delete a[i];
	}
}

int main()
{
	printf("Test 1 ----------------------------------------------------\n");
	test1();
	printf("Test 2 ----------------------------------------------------\n");
	test2();
	return 0;
}
