//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "SharedPointer.hpp"

class A
{
	public:
		A(int _id):ID(_id) { printf("A::Ctor (ID:%d)\n",ID); }
		~A(){ printf("A::Dtor (ID:%d)\n",ID); }
		void Foo() { printf("A::Foo (ID:%d)\n",ID); }
	private:
		int ID;
};


typedef Tools::SharedPointer<A> TAPtr;


//------------------------------------------------------------------------------
// Expected trace :
//
// A::Ctor (ID:1)
// A::Foo (ID:1)
// A::Dtor (ID:1)
//------------------------------------------------------------------------------
void test1()
{
	TAPtr a(new A(1));
	a->Foo();
}

//------------------------------------------------------------------------------
// Expected trace :
//
// A::Ctor (ID:1)
// A::Foo (ID:1)
// A::Foo (ID:1)
// A::Foo (ID:1)
// A::Dtor (ID:1)
//------------------------------------------------------------------------------
void test2()
{
	TAPtr a(new A(1));
	a->Foo();
	{
		TAPtr b = a;
		b->Foo();
	}
	a->Foo();
}

//------------------------------------------------------------------------------
// Expected trace :
//
// A::Ctor (ID:1)
// A::Foo (ID:1)
// A::Foo (ID:1)
// A::Ctor (ID:2)
// A::Foo (ID:2)
// A::Dtor (ID:2)
// A::Foo (ID:1)
// A::Dtor (ID:1)
//------------------------------------------------------------------------------
void test3()
{
	TAPtr a(new A(1));
	a->Foo();
	{
		TAPtr b = a;
		b->Foo();
		b = TAPtr(new A(2));
		b->Foo();
	}
	a->Foo();
}

//------------------------------------------------------------------------------
// Expected trace :
//
// A::Ctor (ID:1)
// A::Foo (ID:1)
// A::Ctor (ID:2)
// A::Foo (ID:2)
// A::Dtor (ID:2)
// A::Dtor (ID:1)
//------------------------------------------------------------------------------
void test4()
{

	TAPtr a;
	if(!a)
		a = TAPtr(new A(1));
	a->Foo();

	TAPtr b;
	if(b == 0)
		b = TAPtr(new A(2));
	b->Foo();
}


int main()
{
	printf("Test 1 --------------------------------\n");
	test1();
	printf("Test 2 --------------------------------\n");
	test2();
	printf("Test 3 --------------------------------\n");
	test3();
	printf("Test 4 --------------------------------\n");
	test4();
	return 0;
}
