//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <cassert>
#include <algorithm>

namespace Tools 
{
	//-------------------------------------------------------------------------
	template <class T>
	inline SharedPointer<T>::SharedPointer():
	object(0),
	objectCounter(0) 
	{

	}
	//-------------------------------------------------------------------------
	template <class T>
	inline SharedPointer<T>::SharedPointer(const SharedPointer<T>& copy) :
	object(copy.object),
	objectCounter(copy.objectCounter) 
	{
		if (objectCounter)
			++(*objectCounter);
	}	
	//-------------------------------------------------------------------------
	template <class T>
	inline SharedPointer<T>::SharedPointer(T* pointer) :
	object(pointer),
	objectCounter(new int(1))
	{

	}
	//-------------------------------------------------------------------------
	template <class T>
	inline SharedPointer<T>::~SharedPointer()
	{
		if (objectCounter && (--(*objectCounter) == 0))	
		{
			delete objectCounter;
			delete object;
		}
	}
	//-------------------------------------------------------------------------
	template <class T>
	inline void SharedPointer<T>::Swap(SharedPointer<T>& pointer)	
	{
		std::swap(object,pointer.object);
		std::swap(objectCounter, pointer.objectCounter);
	}
	//-------------------------------------------------------------------------
	template <class T>
	inline T& SharedPointer<T>::operator *() const	
	{
		assert(object != 0);
		return *object;
	}
	//-------------------------------------------------------------------------
	template <class T>
	inline T* SharedPointer<T>::operator ->() const 
	{
		assert(object != 0);
		return object;
	}
	//-------------------------------------------------------------------------
	template <class T>
	inline const SharedPointer<T>& SharedPointer<T>::operator =(const SharedPointer<T>& pointer)	
	{
		SharedPointer<T>(pointer).Swap(*this);
		return *this;
	}
	//-------------------------------------------------------------------------
	template <class T>
	inline SharedPointer<T>::operator bool()
	{
		return object!=0;
	}
	//-------------------------------------------------------------------------
//	template <class T>
//	inline SharedPointer<T>::operator T*() const	
//	{
//		return object;
//	}
	//-------------------------------------------------------------------------
}
