//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include "SharedPointer.hpp"

class Example
{
	public:
		Example(int _id):ID(_id) 
		{ 
			printf("Example::Ctor (ID:%d)\n",ID); 
		}

		~Example()
		{ 
			printf("Example::Dtor (ID:%d)\n",ID); 
		}
		void Foo() 
		{ 
			printf("Example::Foo (ID:%d)\n",ID); 
		}
	private:
		int ID;
};

typedef Tools::SharedPointer<Example> TExamplePtr;


//------------------------------------------------------------------------------
// Expected trace :
//
// Example::Ctor (ID:1)
// Example::Foo (ID:1)
// Example::Dtor (ID:1)
//------------------------------------------------------------------------------
void test1()
{
	TExamplePtr a(new Example(1));
	a->Foo();
}

//------------------------------------------------------------------------------
// Expected trace :
//
// Example::Ctor (ID:1)
// Example::Foo (ID:1)
// Example::Foo (ID:1)
// Example::Foo (ID:1)
// Example::Dtor (ID:1)
//------------------------------------------------------------------------------
void test2()
{
	TExamplePtr a(new Example(1));
	a->Foo();
	{
		TExamplePtr b = a;
		b->Foo();
	}
	a->Foo();
}

//------------------------------------------------------------------------------
// Expected trace :
//
// Example::Ctor (ID:1)
// Example::Foo (ID:1)
// Example::Foo (ID:1)
// Example::Ctor (ID:2)
// Example::Foo (ID:2)
// Example::Dtor (ID:2)
// Example::Foo (ID:1)
// Example::Dtor (ID:1)
//------------------------------------------------------------------------------
void test3()
{
	TExamplePtr a(new Example(1));
	a->Foo();
	{
		TExamplePtr b = a;
		b->Foo();
		b = TExamplePtr(new Example(2));
		b->Foo();
	}
	a->Foo();
}

//------------------------------------------------------------------------------
// Expected trace :
//
// Example::Ctor (ID:1)
// Example::Foo (ID:1)
// Example::Ctor (ID:2)
// Example::Foo (ID:2)
// Example::Dtor (ID:2)
// Example::Dtor (ID:1)
//------------------------------------------------------------------------------
void test4()
{

	TExamplePtr a;
	if(!a)
		a = TExamplePtr(new Example(1));
	a->Foo();

	TExamplePtr b;
	if(b == 0)
		b = TExamplePtr(new Example(2));
	b->Foo();
}


int main()
{
	printf("Test 1 --------------------------------\n");
	test1();
	printf("Test 2 --------------------------------\n");
	test2();
	printf("Test 3 --------------------------------\n");
	test3();
	printf("Test 4 --------------------------------\n");
	test4();
	return 0;
}
